package com.moduyun.live.utils;

import cn.hutool.crypto.digest.DigestUtil;
import org.springframework.util.StringUtils;

import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.TreeMap;

public class CreateUrlUtils {

    /**
     * 生成推拉流地址
     * @param pushDomainName 推流域名
     * @param pullDomainName 拉流域名
     * @param appName APP名称
     * @param timestamp 失效时间
     * @param streamName 直播流名称
     * @param pushKey 推流密钥
     * @param pullKey 拉流密钥
     * @param templateId 模版id
     * @return
     */
    public static Map<String, String> createUrl(String pushDomainName, String pullDomainName, String appName, Long timestamp, String streamName, String pushKey, String pullKey, String templateId) {
        Map<String, String> map = new LinkedHashMap<>();
        if (!StringUtils.isEmpty(templateId))
            streamName = streamName + "_" + templateId;

        Long newTimestamp = (System.currentTimeMillis() / 1000L) + (60L * timestamp);

        // 推流地址
        String rtmpURL = "rtmp://" + pushDomainName + "/" + appName + "/" + streamName;

        // 拉流地址
        String pullRtmpURL = "rtmp://" + pullDomainName + "/" + appName + "/" + streamName;
        String flvUrl = "http://" + pullDomainName + "/" + appName + "/" + streamName + ".flv";
        String M3U8Url = "http://" + pullDomainName + "/" + appName + "/" + streamName + ".m3u8";
        String udp = "artc://" + pullDomainName + "/" + appName + "/" + streamName;

        // 生成推流拉流地址
        map.put("pushFlowUrl", createAuthKey(rtmpURL, appName, streamName, newTimestamp, pushKey));
        map.put("pullFlowUrl", createAuthKey(pullRtmpURL, appName, streamName, newTimestamp, pullKey));
        map.put("flvFlowUrl", createAuthKey(flvUrl, appName, streamName + ".flv", newTimestamp, pullKey));
        map.put("m3u8FlowUrl", createAuthKey(M3U8Url, appName, streamName + ".m3u8", newTimestamp, pullKey));
        map.put("udpFlowUrl", createAuthKey(udp, appName, streamName, newTimestamp, pullKey));
        return map;
    }

    /**
     * 生成鉴权地址
     * @param url
     * @param timestamp
     * @param key
     * @return
     */
    private static String createAuthKey(String url, String appName, String streamName, Long timestamp, String key) {
        String hashValue = "/" + appName + "/" + streamName + "-" + timestamp + "-0-0-" + key;
        String md5Value = DigestUtil.md5Hex(hashValue);
        String auth_key = timestamp + "-0-0-" + md5Value;
        String newUrl = url + "?auth_key=" + auth_key;
        return newUrl;
    }
}
